#
# get_params.rb
#
# Description: This method is used to find data in VMDB / Prov Object / etc. and be able to store it as a param for Ansible JobTemplate.
#
# jcutter@redhat.com March 2017.
#

begin
  # Change @debug to true to see lots of output in the automation log to help you find the data you are looking for.
  # This method runs frequently and logs a lot if set to true, so keep that in mind.
  #@debug = false
  @debug = true
  $evm.log("info", "Args:    #{MIQ_ARGS.inspect}")
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug
  $evm.object.attributes.sort.each { |k, v| $evm.log("info", "object: #{k}: #{v}") } if @debug
  $evm.parent.attributes.sort.each { |k, v| $evm.log("info", "parent: #{k}: #{v}") } if @debug

  # We will store all our parameters into an array.  After we'll walk the array and set our params into $evm.object.
  params = []

  # Samples of finding data to use as params.  See the log and search for get_params: to find what is available.

  prov = $evm.root["miq_provision"]
  $evm.log(:info, "get_params: prov: #{prov.inspect}") if @debug
  
  vm = prov.vm_template
  $evm.log(:info, "get_params: vm: #{vm.inspect}") if @debug

  ems = vm.ext_management_system
  $evm.log(:info, "get_params: ems: #{ems.inspect}") if @debug

  options = prov.options
  $evm.log(:info, "get_params: options: #{options.inspect}") if @debug

  ws_values = prov.options.fetch(:ws_values, nil)
  $evm.log(:info, "get_params: ws_values: #{ws_values.inspect}") if @debug

  cluster_name = ws_values[:cluster_name]
  $evm.log(:info, "get_params: cluster_name: #{cluster_name}") if @debug

  vm_name = prov.get_option(:vm_name)
  $evm.log(:info, "get_params: vm_name: #{vm_name}") if @debug

  cluster = $evm.vmdb(:ems_cluster).find_by_name(cluster_name)
  $evm.log(:info, "get_params: cluster: #{cluster.name}") if @debug
  
  capsule = ws_values[:capsule]
  $evm.log(:info, "get_params: capsule: #{capsule}") if @debug
  
  actkey = ws_values[:activation_key]
  $evm.log(:info, "get_params: actkey: #{actkey}") if @debug

  tags = prov.get_tags
  $evm.log(:info, "get_params: tags: #{tags.inspect}") if @debug

  # Sample of adding data into params array.
  # Optionally, may consider prefixing all with cf_ so as to avoid clobbering variable names.
  #          param_name=value
  params << "cf_cluster=#{cluster.name}"
  params << "cf_os_version=#{ws_values[:os_version]}"

  # Walk the params array and create $evm.object['param##'] entries, starting at 11 since there are 10 of them in the instance we don't want to overwrite.
  params.each_with_index do |value,index|
    $evm.object["param#{index + 11}"] = value
  end

  $evm.object.attributes.sort.each { |k, v| $evm.log("info", "object after: #{k}: #{v}") } if @debug

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end

