#
# Description: This method checks to see if the VM has been powered off and sends Retirement email
#

# Get vm from root object
vm = $evm.root['vm']

if vm && $evm.get_state_var('vm_soft_retire')
  power_state = vm.attributes['power_state']
  ems = vm.ext_management_system
  $evm.log('info', "VM:<#{vm.name}> on provider:<#{ems.try(:name)} has Power State:<#{power_state}>")
  
  # Get VM Name
  vm_name = vm.name
  
  # Get VM Owner Name and Email
  evm_owner_id = vm.attributes['evm_owner_id']
  owner = nil
  owner = $evm.vmdb('user', evm_owner_id) unless evm_owner_id.nil?
  $evm.log("info", "Inspecting VM Owner: #{owner.inspect}")
  
  userid = owner.userid
  $evm.log(:info, "UserID: #{userid}")     
  data = $evm.instantiate("/Integration/DynamicDialogs/LDAPDialogs/Email?userid=#{userid}") 
  raise 'Error instantiating PScall' if data.blank? 
  email = $evm.root['Email']
  $evm.log(:info, "Email: #{email}") 

  # to_email_address from owner.email then from model if nil
  if vm.tagged_with?("tva_automation", "lab_retirement")
    if email
      to = email.sub('-wa@tva.gov','@tva.gov'),'ITEngLabAutomationTeam@tva.gov'
    else
      to = $evm.object['to_email_address']
    end  
  else  
    if email
      to = email.sub('-wa@tva.gov','@tva.gov'),$evm.object['to_email_address']
    else
      to = $evm.object['to_email_address']
    end
  end
  
######################################
#
# VM has entered Retirement Email
#
######################################

  if %w(off suspended).include?(power_state)
  # Get from_email_address from model unless specified below
  from = nil
  from ||= $evm.object['from_email_address']

  # Get signature from model unless specified below
  signature = nil
  signature ||= $evm.object['signature']

  # email subject
  subject = "VM #{vm_name} has entered retirement"

  # Build email body
  body = "Hello, "
  body += "<br><br>Your virtual machine named [#{vm_name}] has been powered off."
  body += "<br><br>You will have up to 14 days to un-retire this VM. Afterwhich the VM will be deleted."
  body += "<br><br>&quot;https://cloudforms/vm_infra/explorer&quot;"
  body += "<br><br> Thank you,"
  body += "<br> #{signature}"
  
  $evm.log("info", "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
  $evm.execute('send_email', to, from, subject, body)
  # Bump State
  $evm.root['ae_result'] = 'ok'
  elsif power_state == "never"
    # If never then this VM is a template so exit the retirement state machine
    $evm.root['ae_result'] = 'error'
  else
    $evm.root['ae_result']     = 'retry'
    $evm.root['ae_retry_interval'] = '60.seconds'
  end 
end
