#
# Description: This method checks to see if the VM has been powered off or suspended
#

# Get vm from root object
vm = $evm.root['vm']

# Get VM Owner Name and Email
evm_owner_id = vm.attributes['evm_owner_id']
owner = nil
owner = $evm.vmdb('user', evm_owner_id) unless evm_owner_id.nil?

# to_email_address from owner.email then from model if nil
if owner
  email = owner.userid
  $evm.log(:info, "Email: #{email}") if @debug
  data = $evm.instantiate("/Integration/DynamicDialogs/LDAPDialogs/ntid?email=#{email}") 
  raise 'Error instantiating PScall' if data.blank?
  username = $evm.root['userid'].split('@').first.split('-').first
  $evm.log(:info, "Username: #{username}") 
end
ems = vm.ext_management_system if vm

if vm.nil? || ems.nil?
  $evm.log('info', "Skipping check pre retirement for VM:<#{vm.try(:name)}> on EMS:<#{ems.try(:name)}>")
  exit MIQ_OK
end

power_state = vm.power_state
$evm.log('info', "VM:<#{vm.name}> on Provider:<#{ems.name}> has Power State:<#{power_state}>")
retirement_state = vm.retirement_state

# If VM is powered off or suspended exit

if vm.tagged_with?("lifecycle", "soft_retire")
  $evm.log('info', "Powering Off VM <#{vm.name}> in provider <#{ems.try(:name)}>")
  vm.stop
  $evm.log("info", "Extending retirement for VM: <#{vm.name}>")
  vm.retires_on = Time.now + 14.days
  $evm.log("info", "Tagging VM <#{vm.name}> for full retirement")
  vm.tag_unassign('lifecycle/soft_retire')
  vm.tag_assign('lifecycle/retire_full')
  $evm.log("info","Retirement Warning reset for VM: <#{vm.name}>")
  vm.retirement_warn = 2
  $evm.set_state_var('vm_soft_retire', true)
elsif vm.tagged_with?("lifecycle", "retire_full") && vm.tagged_with?("tva_automation", "lab_retirement")
  #Call PScall method to run ARPcloud-Retirement.ps1 script
  $evm.log(:info, "Instantiating PowerShell call") if @debug
  #$evm.instantiate("/Integration/CommonMethods/Powershell/PScallDev?script=C:\\vCO\\Scripts\\LabAutomation\\LabRetirement.ps1&value1=-Name '#{vm.name}' -NTID '#{username}'")
  $evm.instantiate("/Integration/CommonMethods/Powershell/PScall?script=C:\\vCO\\Scripts\\ARPNET\\ArpCloud\\ARPcloud-Retirement.ps1&value1=-Name '#{vm.name}' -NTID '#{username}'")
  $evm.set_state_var('vm_removed_from_provider', true)
  # Bump State
  $evm.root['ae_result'] = 'ok'
elsif vm.tagged_with?("lifecycle", "retire_full")
  #Call PScall method to run ARPcloud-Retirement.ps1 script
  $evm.log(:info, "Instantiating PowerShell call") if @debug
  $evm.instantiate("/Integration/CommonMethods/Powershell/PScall?script=C:\\vCO\\Scripts\\ARPNET\\ArpCloud\\ARPcloud-Retirement.ps1&value1=-Name '#{vm.name}' -NTID '#{username}'")
  $evm.set_state_var('vm_removed_from_provider', true)
  # Bump State
  $evm.root['ae_result'] = 'ok'
elsif power_state == "never"
  # If never then this VM is a template so exit the retirement state machine
  $evm.root['ae_result'] = 'error'
else
  $evm.root['ae_result'] = 'retry'
  $evm.root['ae_retry_interval'] = '60.seconds'
end
