#
# PScallLong.rb
#
# Description: Run a PowerShell job that takes over 10 minutes to complete.
#
# jcutter@redhat.com March 2017.
#

begin

  def rm_files(files)
    files.each do |file|
      $evm.log(:info, "Removing file #{file}")
      File.delete(file) if File.exist?(file)
    end
  end

  def get_file_contents(file)
    if ! File.exists?(file)
      contents = 'File does not exist.'
    elsif File.zero?(file)
      contents = 'File is empty.'
    else
      File.exists?(file) && ! File.zero?(file)
      contents = IO.read(file)
    end
    contents
  end

  @debug = true
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug

  @task = $evm.root['service_template_provision_task']

  # Setup tmp directory if doesn't exist.
  tmp_dir = '/tmp/.cloudforms_powershell'
  Dir.mkdir(tmp_dir) unless File.exists?(tmp_dir)
  File.chmod(0600, tmp_dir)

  # Tmp files.
  unique_id = $evm.root['service_template_provision_task_id']
  rb_script = "#{tmp_dir}/cf_pscall_#{unique_id}.rb"
  rb_out = "#{tmp_dir}/cf_pscall_#{unique_id}.rb_out"
  rb_err = "#{tmp_dir}/cf_pscall_#{unique_id}.rb_err"
  sh_out = "#{tmp_dir}/cf_pscall_#{unique_id}.sh_out"
  sh_err = "#{tmp_dir}/cf_pscall_#{unique_id}.sh_err"
  pid_file = "#{tmp_dir}/cf_pscall_#{unique_id}.pid"

  # Collect up specifics for this run from $evm.
  script = $evm.object['script']
  script.gsub!('\\','\\\\\\')
  value1 = $evm.object['value1']
  hostname = $evm.object['hostname']
  $evm.log(:info, "script: #{script} value: #{value1} hostname: #{hostname}")

  $evm.log(:info, "Credentials to connect Windows VM")
  username = $evm.object['username']
  password = $evm.object.decrypt('password')

  url_params = {
      :ipaddress => hostname,
      :port      => 5985
  }
  connect_params = {
      :user         => username,
      :pass         => password,
      :disable_sspi => true
  }

  url = "http://#{url_params[:ipaddress]}:#{url_params[:port]}/wsman"
  $evm.log("info", "Connecting to WinRM on URL: #{url}")

  # ruby_script to be written to a file and run via system call with nohup.

ruby_script = <<-EOF
#!/usr/bin/env ruby
require 'winrm'
require 'json'

def write_results(file, results)
  unless results.blank?
    file = File.open("\#{file}", "w")
    File.chmod(0600, file)
    file.puts(results)
    file.close
  end
end

PS_SCRIPT = <<-PS_SCRIPT
& '#{script}' #{value1}
PS_SCRIPT

begin
  status = Timeout::timeout(120*60) do
    winrm   = WinRM::WinRMWebService.new('#{url}', :ssl, #{connect_params})
    results = winrm.run_powershell_script(PS_SCRIPT)

    errors = results[:data].collect { |d| d[:stderr] }.join
    write_results('#{rb_err}', errors) unless errors.blank?

    data = results[:data].collect { |d| d[:stdout] }.join
    write_results('#{rb_out}', data)
  end
# Have to rescue Exception instead of Timeout::Error because Ruby Timeout poorly kills the thread causing an Exception to be thrown.
# This means we're also rescuing other potential issues, but still e-mailing them to the right place at least.
rescue Exception

  require 'socket'

  def send_email(to,opts={})
    opts[:server]      ||= 'localhost'
    opts[:from]        ||= 'vmwareadmins@tva.gov'
    opts[:from_alias]  ||= 'VMware Admins'
    opts[:subject]     ||= 'subject blank'
    opts[:body]        ||= 'body blank'

    msg = <<END_OF_MESSAGE
From: \#{opts[:from_alias]} <\#{opts[:from]}>
To: <\#{to}>
Subject: \#{opts[:subject]}

\#{opts[:body]}
END_OF_MESSAGE

    require 'net/smtp'
    Net::SMTP.start(opts[:server]) do |smtp|
      smtp.send_message msg, opts[:from], to
    end
  end

  to = 'AutomationWorkflowAdmins@tva.gov'
  subject = "Timeout running \#{$PROGRAM_NAME} on \#{Socket.gethostname}"
  body = <<-BODY

\#{subject}

PowerShell Command:

& '#{script}' #{value1}

BODY
  send_email to, :subject => subject, :body => body

end

EOF

  # Write ruby_script to a file.
  fh = File.open(rb_script, 'w')
  File.chmod(0700, rb_script)
  fh.puts(ruby_script)
  fh.close
  $evm.log(:info, "Wrote ruby_script to #{rb_script}")

  # Make a system call to run ruby_script with nohup.
  $evm.log(:info, "Running ruby_script #{rb_script} with nohup")
  system("nohup #{rb_script} > #{sh_out} 2>#{sh_err} & echo $! > #{pid_file}")
  pid = IO.read(pid_file).chomp
  # Sleep a small amount of time to check that the WinRM job was submitted successfully.
  $evm.log(:info, "Waiting for ruby_script #{rb_script} to be initiate job with WinRM")
  sleep 15

  # 1. Errored out (this is what we want to catch as rest of error handling will occur on the PowerShell side).
  # 2. Exited cleanly (surprising?)
  # 3. Still running (good!)

  if ! ( Process.getpgid(pid.to_i) rescue nil )
    # Process is not still running.
    $evm.log(:info, "Process #{pid} for ruby_script #{rb_script} is not running.")
    # If sh_out does not exist or sh_err has contents, something went badly wrong starting ruby_script.
    if ! File.exists?(sh_out) || ! File.zero?(sh_err)
      error = "Failed to run ruby_script #{rb_script} due to shell related issues"
    # If rb_out and rb_err do not exist or if rb_out is empty.
    elsif ! File.exists?(rb_out) || File.zero?(rb_out)
      error = "Failed to run ruby_script #{rb_script} due to Ruby / WinRM related issues"
    else
      $evm.log(:info, "No indicators of a failure for #{pid} / #{rb_script} were found.")
      error = nil
    end
    if error
      # Log error
      $evm.log(:error, error) unless error.blank?
      # email error
      to = $evm.object['to_email_address']
      from = $evm.object['from_email_address']
      signature = $evm.object['signature']
      subject = "CloudForms Alert: Error running #{$evm.current_method}"
      body = <<-EOF
    
#{error}

CloudForms method #{$evm.current_method} encountered the error above attempting to run the following:

Catalog Item: #{$evm.root['object_name']}
PowerShell Script: & '#{script}' #{value1}
CloudForms Appliance: #{$evm.root['miq_server'].hostname}
service_template_provision_task_id: #{unique_id}

Contents of shell stdout file:
#{get_file_contents(sh_out)}

Contents of shell stderr file:
#{get_file_contents(sh_err)}

Contents of WinRM data:
#{get_file_contents(rb_out)}

Contents of WinRM errors:
#{get_file_contents(rb_err)}

--
#{signature}

EOF
      body.gsub!(/$/, '<br>')
      raise 'Unable to send e-mail because cannot find to_email_address and/or from_email_address' if to.blank? || from.blank?
      $evm.log(:info, "Emailing following error to #{to}:
#{body.gsub(/<br>/, '')}")
      $evm.execute(:send_email, to, from, subject, body)
    end

  else
    $evm.log(:info, "ruby_script is running successfully")
  end

  # Clean up tmp files.
  rm_files [ rb_script, rb_out, rb_err, sh_out, sh_err, pid_file ]

  exit MIQ_OK

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  @task['status'] = 'Error' if @task
  @task.finished("#{err}") if @task
  @service.remove_from_vmdb if @service
  exit MIQ_ABORT
end
