begin

  require 'rest_client'
  require 'json'
  require 'date'

def call_api(payload)

  server    = $evm.object['hostname']
  username  = $evm.object['username']
  password  = $evm.object.decrypt('password')
  $evm.log(:info, "server: #{server}, username: #{username}") if @debug

  url = "https://#{server}/api/v1/job_templates/165/schedules/"

  params = {
    :method   => :post,
    :url      => url,
    :user     => username,
    :password => password,
    :headers  => {
      :content_type  => :json,
      :accept        => :json
    },
    :timeout => 90,
    :payload => payload,
    :verify_ssl  => false
  }

  # call the REST service
  begin
    response = RestClient::Request.new(params).execute
    rescue => e
    $evm.log(:error, "REST response: #{e.response.body.inspect}") if @debug
    raise e
  end

  unless response.code == 200 || response.code == 201 || response.code == 202
      $evm.log(:error, "Error calling REST API")
      $evm.log(:info, "Response Code: <#{response.code}>  Response: <#{response.inspect}>")
  end
    $evm.log(:info, "API Response Data: #{response.inspect}") if @debug
end

  request_id = $evm.object['request_id']
  schedule = $evm.object['scheduled_time']
  source_database = $evm.object['source_database']
  destination_host = $evm.object['destination_host']
  destination_sid = $evm.object['destination_sid']
  export_type = $evm.object['export_type']
  source_export = $evm.object['source_export']
  
  start_time = DateTime.strptime(schedule, '%Y-%m-%dT%H:%M:%SZ').strftime("%Y%m%dT%H%M%SZ")

  payload = {
    "name": "Cloudforms Schedule Request - #{request_id}",
    "description": "Cloudforms Schedule Request - #{request_id}",
    "enabled": true,
    "rrule": "DTSTART:#{start_time} RRULE:FREQ=DAILY;INTERVAL=1;COUNT=1",
    "extra_data": {
       "source_database": source_database,
       "destination_host": destination_host,
       "destination_sid": destination_sid,
       "export_type": export_type,
       "source_export": source_export
    }
  }.to_json
  
  call_api(payload)
  exit MIQ_OK
  
rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT  
end
