#
# list_patch_windows.rb
#
# Description: List valid patch windows from automate database instances based on environment.
#              Using instances in instance_path as a database, determine the environment from the
#              dialog and show only valid patch windows for that environment.
#
# jcutter@redhat.com March 7, 2017.
#

@debug = true
instances_path = '/TVA_Variables/PatchWindows/PatchWindows/*'

$evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug

# Get the environment from the dialog.
environment =  $evm.root['dialog_tag_1_environment'].first.description[0] rescue nil
$evm.log(:info, "environment: #{environment}") if @debug

# If environment is Sandbox, look for L.  Rest should be consistent.
environment = 'L' if environment == 'S'

# Get all the instances.
instances = $evm.instance_find(instances_path)
$evm.log(:info, "all instances: #{instances.inspect}") if @debug

# Get just the ones that match our environment.
instances = instances.select { |k,v| k[0] == environment }
$evm.log(:info, "selected instances: #{instances.inspect}") if @debug

# Convert hash of hashes to hash for dialog_values.
dialog_values = {}
instances.each_key do |i|
  dialog_values[i] = "#{i}: #{instances[i]['description']}"
end
$evm.log(:info, "values = #{dialog_values.inspect} ")

dialog_field                  = $evm.object
dialog_field['sort_by']       = 'value'
dialog_field['data_type']     = 'string'
dialog_field['values']        = dialog_values

dialog_field
