#
# capsule_select.rb
#
# Description: Select Satellite capsule server for newly provisioned Linux hosts
#
# ltdoux@tva.gov March 2017
#

begin
  @debug = true
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug
  $evm.object.attributes.sort.each { |k, v| $evm.log("info", "object: #{k}: #{v}") } if @debug
  request = $evm.root['miq_provision_request']
  
  # Collect info from other dialogs.
  environment =  $evm.root['dialog_tag_1_environment'].first.name rescue nil
  $evm.log(:info, "environment: #{environment}") if @debug
  role =  $evm.root['dialog_tag_1_role'].first.name rescue nil
  $evm.log(:info, "role: #{role}") if @debug
  city =  $evm.root['dialog_tag_1_city'].first.name rescue nil
  $evm.log(:info, "city: #{city}") if @debug
  replicated =  $evm.root['dialog_tag_1_replicated'].first.name rescue nil
  $evm.log(:info, "replicated: #{replicated}") if @debug
  cluster = $evm.root['dialog_option_1_cluster_name']
  $evm.log(:info, "cluster: #{cluster}") if @debug
  os = $evm.root['dialog_option_1_os_version']
  $evm.log(:info, "os: #{os}") if @debug
  exit MIQ_OK if environment.blank? || role.blank? || city.blank? || replicated.blank?

  # Select activation key

  # If we're in the lab, use one of the lab keys
  if cluster == 'VPLEX Unified Datacenter'
    if os == 'rhel6'
      activation_key = 'Lab RHEL 6'
    else
      activation_key = 'Lab RHEL 7'
    end
  elsif environment == 'sandbox'
    if os == 'rhel6'
      activation_key = 'Sandbox RHEL 6'
    else
      activation_key = 'Sandbox RHEL 7'
    end
  elsif environment == 'dev'
    if os == 'rhel6'
      activation_key = 'Dev RHEL 6'
    else
      activation_key = 'Dev RHEL 7'
    end
  elsif environment == 'acpt'
    if os == 'rhel6'
      activation_key = 'Acpt RHEL 6'
    else
      activation_key = 'Acpt RHEL 7'
    end
  else 
    if os == 'rhel6'
      activation_key = 'Prod RHEL 6'
    else
      activation_key = 'Prod RHEL 7'
    end
  end

  $evm.log(:info, "Satellite Activation Key: #{activation_key}") if @debug
 
  list_values = {
  'required' => true,
  'protected' => false,
  'read_only' => true,
  'value' => activation_key,
  }
  list_values.each do |key, value|
  $evm.object[key] = value
  end
  
  $evm.root['param12'] = activation_key
  
rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
