#
# capsule_select.rb
#
# Description: Select Satellite capsule server for newly provisioned Linux hosts
#
# ltdoux@tva.gov March 2017
#

begin
  @debug = true
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug
  $evm.object.attributes.sort.each { |k, v| $evm.log("info", "object: #{k}: #{v}") } if @debug
  request = $evm.root['miq_provision_request']
  
  # Collect info from other dialogs.
  environment =  $evm.root['dialog_tag_1_environment'].first.name rescue nil
  $evm.log(:info, "environment: #{environment}") if @debug
  role =  $evm.root['dialog_tag_1_role'].first.name rescue nil
  $evm.log(:info, "role: #{role}") if @debug
  city =  $evm.root['dialog_tag_1_city'].first.name rescue nil
  $evm.log(:info, "city: #{city}") if @debug
  replicated =  $evm.root['dialog_tag_1_replicated'].first.name rescue nil
  $evm.log(:info, "replicated: #{replicated}") if @debug
  cluster = $evm.root['dialog_option_1_cluster_name']
  $evm.log(:info, "cluster: #{cluster}") if @debug
  exit MIQ_OK if environment.blank? || role.blank? || city.blank? || replicated.blank?

  # Select capsule

  # If we're a database server, go to dbsatp
  if role == 'db'
    capsule_name = 'dbsatp.main.tva.gov'
  elsif cluster == 'VPLEX Unified Datacenter'
    capsule_name = 'chatlabsatcap.main.tva.gov'
  elsif city == 'knx'
    capsule_name = 'knxpusat.main.tva.gov'
  else
    capsule_name = 'chapusat.main.tva.gov'
  end

  $evm.log(:info, "Satellite Capsule: #{capsule_name}") if @debug
 
  list_values = {
  'required' => true,
  'protected' => false,
  'read_only' => true,
  'value' => capsule_name,
  }
  list_values.each do |key, value|
  $evm.object[key] = value
  end
  
  $evm.root['param11'] = capsule_name
  
rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
