#
# list_valid_clusters.rb
#
# Description: List valid clusters which are valid for this provision based upon other dialogs.
#
# jcutter@redhat.com March 2017.
#

begin
  @debug = true
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug
  $evm.object.attributes.sort.each { |k, v| $evm.log("info", "object: #{k}: #{v}") } if @debug

  # environment = prod | acpt && role = db then
  # Load cluster selection criteria from automate database.

  # Collect info from other dialogs.
  environment =  $evm.root['dialog_tag_1_environment'].first.name rescue nil
  $evm.log(:info, "environment: #{environment}") if @debug
  role =  $evm.root['dialog_tag_1_role'].first.name rescue nil
  $evm.log(:info, "role: #{role}") if @debug
  city =  $evm.root['dialog_tag_1_city'].first.name rescue nil
  $evm.log(:info, "city: #{city}") if @debug
  replicated =  $evm.root['dialog_tag_1_replicated'].first.name rescue nil
  $evm.log(:info, "replicated: #{replicated}") if @debug
  exit MIQ_OK if environment.blank? || role.blank? || city.blank? || replicated.blank?

  # Select cluster.

  # If we're db and prod or acpt then we'll use what we find in the instance for db_clusters.
  if role == 'db' && ( environment == 'prod' || environment == 'acpt' )
    cluster_names = $evm.root['db_clusters']
    raise 'db_clusters not found in instance.' if cluster_names.blank?
  else
    # Load cluster selection criteria from automate database.
    cluster_instances_path = '/TVA_Variables/SelectionCriteria/ClusterSelection/*'
    cluster_selections = $evm.instance_find(cluster_instances_path)
    $evm.log(:info, "cluster_selections: #{cluster_selections.inspect}") if @debug
    # Determine valid clusters.
    valid_clusters = cluster_selections.select { |k,v| v['environment'] == environment && v['role'] == role && v['city'] == city }
    valid_clusters = valid_clusters.select { |k,v| k =~ /^udc_/ } if replicated == 'yes' # If replicated is yes, the instance name must start with udc_
    valid_clusters = valid_clusters.select { |k,v| k !~ /^udc_/ } if replicated != 'yes' # If replicated is no, the instance name must not start with udc_
    $evm.log(:info, "valid_clusters: #{valid_clusters.inspect}") if @debug
    #cluster_names = [ valid_clusters[valid_clusters.keys.first]['cluster_name'] ] unless valid_clusters.blank?
    cluster_names = valid_clusters.collect { |k,v| v['cluster_name'] } unless valid_clusters.blank?
    $evm.log(:info, "cluster_names: #{cluster_names}") if @debug
  end


  dialog_values = {}
  cluster_names.each { |cluster| dialog_values[cluster.gsub('/', '%2f')] = cluster } unless cluster_names.blank?
  $evm.log(:info, "values = #{dialog_values.inspect} ")

  dialog_field                  = $evm.object
  dialog_field['sort_by']       = 'value'
  dialog_field['data_type']     = 'string'
  $evm.object['default_value']  = dialog_values.keys.sort.first
  dialog_field['values']        = dialog_values

  dialog_field

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end

