#
# acquire_ip.rb
#
# Description: Get the IP from Infoblox using PowerShell integration.
#
# Intended to be called from provisioning state machine.
#
# jcutter@redhat.com March 2017.
#

begin

  require 'ipaddr'

  @debug = true
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug
  $evm.object.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug

  # Get provisioning object
  prov = $evm.root["miq_provision"]
  $evm.log(:info, "Provision:<#{prov.id}> Request:<#{prov.miq_provision_request.id}> Type:<#{prov.type}>")

  # Get vm name.
  name = prov.get_option(:vm_name).to_s.strip

  # Get the destination network from provisioning.
  
  vlan = prov.get_option(:vlan).to_s
  $evm.log(:info, "vlan: #{vlan}") if @debug
  raise 'Unable to determine destination network (vlan) from prov object.' if vlan.blank?
  $evm.log(:info, "vlan: #{vlan}")

  # Network names can be in the format of:
  # FOO-BAR-BIZ-1.1.1.X => means a /24
  # FOO-BAR-BIZ-1.1.1.0/25
  cidr = vlan.split('-').last
  cidr = cidr.rpartition('.').first + ".0/24" if cidr =~ /\.X$/i
  $evm.log(:info, "cidr: #{cidr}")

  # Determine the subnet mask.
  net_obj = IPAddr.new(cidr)
  subnet_mask = net_obj.inspect.split('/').last.gsub('>', '') # netaddr not on appliance by default and ipaddr doesn't have a method to pull mask.
  $evm.log(:info, "subnet_mask: #{subnet_mask}")

  # Determine the default gateway.
  gateway = net_obj.succ.to_s # Gateway is one more than the network address.
  $evm.log(:info, "gateway: #{gateway}")

  # Call PowerShell method to run InfoBlox script to get next IP and register it in DNS.
  # C:\vCO\Functions\Infoblox\Infoblox-CreateHostRecord.ps1' -Name labtest99 -Network 192.168.1.0/24
  result = $evm.instantiate("/Integration/CommonMethods/PowerShell/PScall?script=C:\\vCO\\Functions\\Infoblox\\Infoblox-CreateHostRecord.ps1&value1=-Name #{name} -Network #{cidr}")
  raise 'Error instantiating PScall' if result.blank?
  $evm.log(:info, "result: #{result.inspect}")
  my_ip = $evm.root['ps_data'].match(/(^\d{1,3}\.\d{1,3}\.\d{1,3}\.\d{1,3})/).captures.first
  raise 'Failed to get IP address from Infoblox' if my_ip.blank?
  $evm.log(:info, "my_ip: #{my_ip}")

  # Store the results into the prov object.
  prov.set_nic_settings(0, {:ip_addr => my_ip, :subnet_mask => subnet_mask, :gateway => gateway, :addr_mode => ["static", "Static"]})

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
