#
# create_ticket.rb
#
# Description: Creates tickets using PowerShell.
#
# 

begin
 
  @debug = true
  #$evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug
  #$evm.object.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug

  # Get provisioning object
  prov = $evm.root["miq_provision"]
  $evm.log(:info, "Provision:<#{prov.id}> Request:<#{prov.miq_provision_request.id}> Type:<#{prov.type}>")
  
  # Collect required values from ws_values
  ws_values = prov.options.fetch(:ws_values, nil)
  $evm.log(:info, "get_params: ws_values: #{ws_values.inspect}") if @debug
  
  # Exit, if creation of tickets not required
  test = ws_values[:test].to_s
  $evm.log(:info, "Tickets: #{test}")
  if test == 'Yes'
    $evm.log(:info, "Skipping creation of tickets") if @debug
    $evm.root['ae_next_state'] = 'RegisterDHCP'
    exit MIQ_OK   
  end
  
  # Collect required parameters to create tickets
  server_name = prov.get_option(:vm_name).to_s + '.main.tva.gov'
  $evm.log(:info, "server_name: #{server_name}")
  cluster_name = ws_values[:cluster_name].to_s
  $evm.log(:info, "cluster_name: #{cluster_name}")
  cluster = $evm.vmdb(:ems_cluster).find_by_name(cluster_name)
  raise "cluster #{cluster_name} not found!" if cluster.blank?
  $evm.log(:info, "Cluster: #{cluster.name}")
  ems_id = cluster.ems_id
  $evm.log(:info, "ems_id: #{ems_id}")
  vcenter_name = $evm.vmdb(:ems).find_by_id(ems_id).name
  $evm.log(:info, "vcenter_name: #{vcenter_name}")
  datacenter_name = ''
  $evm.vmdb(:ems).find_by_name("#{vcenter_name}").ems_clusters.each {|cluster| datacenter_name = cluster.v_parent_datacenter if cluster.name == "#{cluster_name}"}.to_s
   datacenter_name = datacenter_name.gsub('%2f','/')
  $evm.log(:info, "datacenter_name: #{datacenter_name}")
  
  city = ws_values[:city].to_s  
  if city == 'cha'
    city_name = 'Chattanooga'
  elsif city == 'knx'
    city_name = 'Knoxville'
  elsif city == 'msh'
    city_name = 'Muscle Shoals'
  else
    city_name = 'Nashville'
  end
  $evm.log(:info, "city: #{city_name}")
  
  environment = ws_values[:environment].to_s
  if environment == 'prod'
    environment_name = 'Production'
  elsif environment == 'acpt'
    environment_name = 'Acceptance'
  else
    environment_name = 'Development'
  end  
  $evm.log(:info, "environment: #{environment_name}")
  
  change_id = ws_values[:changeid].to_s
  $evm.log(:info, "change_id : #{change_id}")
  description = ws_values[:description].to_s
  $evm.log(:info, "description: #{description}")
  sox = ws_values[:sox].to_s
  $evm.log(:info, "sox : #{sox}") 
  
  role = ws_values[:role].to_s
  $evm.log(:info, "role : #{role}") 
  if role == 'db'
    database = 'Oracle'
  else
    database = 'N/A'
  end
  $evm.log(:info, "Database : #{database}") 
  
  email = ws_values[:user_id].to_s
  $evm.log(:info, "Email: #{email}") if @debug    
  data = $evm.instantiate("/Integration/DynamicDialogs/LDAPDialogs/ntid?email=#{email}") 
  raise 'Error instantiating PScall' if data.blank? 
  username = $evm.root['userid'].split('@').first.split('-').first
  $evm.log(:info, "Username: #{username}") if @debug
  
  # Call PowerShell method to create tickets.
  #result = $evm.instantiate("/Integration/CommonMethods/PowerShell/PScall?script=C:\\vCO\\Scripts\\Compute\\Linux\\LinuxServerDeployTickets\\LinuxServerDeployTickets.ps1&value1=-Name '#{server_name}' -vCenter '#{vcenter_name}' -Datacenter '#{datacenter_name}' -City '#{city_name}' -ChangeID '#{change_id}' -Description '#{description}' -Timezone 'EST' -Environment '#{environment_name}' -Database '#{database}' -SoxNerc '#{sox}' -BackupType 'Standard' -NTID '#{username}' -Test 'Yes'")
  #raise 'Error instantiating PScall' if result.blank?
  #$evm.log(:info, "result: #{result.inspect}")
  $evm.log(:info, "Collecting parameters to launch Ansible Tower") if @debug
  $evm.root['param24'] = "powershell_cmd= C:\\vCO\\Scripts\\Compute\\Linux\\LinuxServerDeployTickets\\LinuxServerDeployTickets.ps1 -Name '#{server_name}' -vCenter '#{vcenter_name}' -Datacenter '#{datacenter_name}' -City '#{city_name}' -ChangeID '#{change_id}' -Description '#{description}' -Timezone 'EST' -Environment '#{environment_name}' -Database '#{database}' -SoxNerc '#{sox}' -BackupType 'Standard' -NTID '#{username}' -Test 'No'"

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
