# Author: Jeffrey Cutter (jcutter@redhat.com)
#
# Description: Call a vRA workflow to update uCMDB.
#

begin

  require 'rest_client'
  require 'json'

  def call_api(payload, guid)

    # get the server fqdn/ip and port number to send the request to
    server      = $evm.object['hostname']
    username    = $evm.object['username']
    password    = $evm.object.decrypt('password')
    $evm.log(:info, "server: #{server}, username: #{username}") if @debug

    # build the URL for the REST request
    url = "http://#{server}:8281/vco/api/workflows/#{guid}/executions"

    $evm.log(:info, "Call vRA API URL: <#{url}>") if @debug
    $evm.log(:info, "payload: #{payload.inspect}") if @debug

    # build the REST request
    params = {
        :method   => :post,
        :url      => url,
        :user     => username,
        :password => password,
        :headers  => {
            :content_type  => :json,
            :accept        => :json
        },
        :timeout => 90,
        :payload => payload,
        :verify_ssl  => false
    }

    # call the REST service
    begin
      response = RestClient::Request.new(params).execute
    rescue => e
      $evm.log(:error, "REST response: #{e.response.body.inspect}") if @debug
      raise e
    end

    unless response.code == 200 || response.code == 201 || response.code == 202
      $evm.log(:error, "Error calling REST API")
      $evm.log(:info, "Response Code: <#{response.code}>  Response: <#{response.inspect}>")
    end
    $evm.log(:info, "API Response Data: #{response.inspect}") if @debug
  end

  #######
  # BEGIN HERE
  #######

  @debug = true
  #$evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug
  #$evm.object.attributes.sort.each { |k, v| $evm.log("info", "object: #{k}: #{v}") } if @debug

  guid = '8a67d14a-0d4d-4f3c-93c6-89b4b66e66b7'

  #test_prov = $evm.root['dialog_option_1_Test']
  #raise 'dialog_option_1_test not specified' if test_prov.blank?

  #if test_prov != 'No'
  #  $evm.log(:info, "dialog_option_1_Test is not specified as No, therefore, assuming this is a test provision and skipping uCMDB call.")
  #  exit MIQ_OK
  #end
  
  #server_name = $evm.root['dialog_option_1_name']
  #server_name ||= $evm.root['dialog_option_1_vm_name']
  #change_id = $evm.root['dialog_option_1_changeid']
  #os = $evm.root['dialog_option_1_os']
  #os ||= $evm.root['dialog_option_1_os_version']
  server_name = 'cmtkappd1'
  change_id = 'C2000502'
  os = 'rhel7'
  raise 'Required parameter not found!' if server_name.blank? || change_id.blank? || os.blank?

  payload = {"parameters":
    [
                {
                                "value": {"string":{ "value": server_name}},
                                "type": "string",
                                "name": "serverName",
                                "scope": "local"
                },
                {
                                "value": {"string":{ "value": change_id}},
                                "type": "string",
                                "name": "ChangeID",
                                "scope": "local"
                },
                {
                                "value": {"string":{ "value": os}},
                                "type": "string",
                                "name": "OS",
                                "scope": "local"
                }
    ]
  }.to_json

  call_api(payload, guid)

  exit MIQ_OK

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
