#
# remove_empty_services: Cleanup not new services which do not contain any instances or VMs.
#
# Author: jcutter@redhat.com
#

begin

  def get_matching_objects(random_id, object_type)
	matches = $evm.vmdb(object_type).where("name ~ '-#{random_id}'")
	matches.select! {|i| i.archived != true} if object_type == :vm
	$evm.log(:info, "Found #{matches.count} #{object_type}(s).")
	matches
  end
  
  @debug = true

  $evm.log("info", "Starting #{$evm.current_method}")

  $evm.vmdb('service').all.each do |service|
    #$evm.log(:info, "Service #{service.name} has #{service.vms.count} instance(s).") if @debug
    next unless service.vms.count == 0
    # Service has no instances associated.
    # Only remove if service is older than a day.
    if service.created_at.to_time < 1.day.ago
      $evm.log(:info, "Service #{service.name} created on #{service.created_at.to_time} has no matching instances or stacks and is older than a day, deleting.")
      service.remove_from_vmdb
    else
      # Service is less than a day.  Leave it alone.
      $evm.log(:info, "Service #{service.name} created on #{service.created_at.to_time} has no instances but is less than a day old, leaving.")
    end
  end

  $evm.log(:info, "Ending #{$evm.current_method}")
  exit MIQ_OK

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
