#
# tva_edp.rb
#
# Description: <Method description here>
#

begin

  def log(level, msg, update_message = false)
    $evm.log(level, "#{msg}")
    @task.message = msg if @task && (update_message || level == 'error')
  end

  def yaml_data(option)
    @task.get_option(option).nil? ? nil : YAML.load(@task.get_option(option))
  end

  # check to ensure that dialog_parser has ran
  def parsed_dialog_information
    dialog_options_hash = yaml_data(:parsed_dialog_options)
    dialog_tags_hash = yaml_data(:parsed_dialog_tags)
    if dialog_options_hash.blank? && dialog_tags_hash.blank?
      log(:info, "Instantiating dialog_parser to populate dialog options")
      $evm.instantiate('/Service/Provisioning/StateMachines/Methods/DialogParser')
      dialog_options_hash = yaml_data(:parsed_dialog_options)
      dialog_tags_hash = yaml_data(:parsed_dialog_tags)
      raise 'Error loading dialog options' if dialog_options_hash.blank? && dialog_tags_hash.blank?
    end
    log(:info, "dialog_options_hash: #{dialog_options_hash.inspect}")
    log(:info, "dialog_tags_hash: #{dialog_tags_hash.inspect}")
    return dialog_options_hash, fix_dialog_tags_hash(dialog_tags_hash)
  end

  # fix_dialog_tags_hash to allow for support of Tag Control items in dialogs.  Previously, the Tag Control would
  # push values in as an array, which is not supported.  The fix_dialog_tag_hash parsed the dialog_tags_hash and changes
  # all array values to strings.
  def fix_dialog_tags_hash(dialog_tags_hash)
    unless dialog_tags_hash.empty?
      dialog_tags_hash.each do |build_num,build_tags_hash|
        build_tags_hash.each do |k,v|
          if v.is_a?(Array)
            log(:info, "fix_dialog_tags_hash: Build #{build_num}: updating key <#{k}> with array value <#{v}> to <#{v.first}>")
            build_tags_hash[k] = v.first if v.is_a?(Array)
          end
        end
      end
    end
    dialog_tags_hash
  end

  @debug = true
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug

  @task = $evm.root['service_template_provision_task']
  @service = @task.destination
  log(:info, "Service: #{@service.name} id: #{@service.id} tasks: #{@task.miq_request_tasks.count}")

  dialog_options_hash, dialog_tags_hash = parsed_dialog_information

  # Massage the option_1 items into our arguments for PowerShell.
  ps_args = []
  dialog_options_hash[1].each do |k,v|
    ps_args << "#{k}: '#{v}'"
  end

  if ps_args
    $evm.log(:info, "ps_args: #{ps_args.join(' ')}") if @debug
    #email options
    to = 'tkopuru@tva.gov'
    from = 'VMwareAdmins@tva.gov'
    signature = 'VMwareAdmins'
    subject = "TVA Engineering Design Package"
    body = <<-EOF

#Data from EDP Ctatalog

#{ps_args}
--
#{signature}

EOF
    body.gsub!(/$/, '<br>')
    raise 'Unable to send e-mail because cannot find to_email_address and/or from_email_address' if to.blank? || from.blank?
    $evm.log(:info, "Emailing data #{to}")
    $evm.execute(:send_email, to, from, subject, body)
  else
    $evm.log(:info, "ps_args is empty")
  end

rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  @task['status'] = 'Error' if @task
  @task.finished("#{err}") if @task
  @service.remove_from_vmdb if @service
  exit MIQ_ABORT
end
