#
# Description: This method is used to email the provision requester that
# the Service provisioning request has been approved
#
# Events: request_approved
#
# Model Notes:
# 1. to_email_address - used to specify an email address in the case where the
#    requester does not have a valid email address. To specify more than one email
#    address separate email address with commas. (I.e. admin@example.com,user@example.com)
# 2. from_email_address - used to specify an email address in the event the
#    requester replies to the email
# 3. signature - used to stamp the email with a custom signature
#

def emailrequester(miq_request, appliance, catalog_item)
  $evm.log('info', "Requester email logic starting")

  # Get requester object
  requester = miq_request.requester

  # Get requester email else set to nil
  userid = $evm.root['user'].userid  
  data = $evm.instantiate("/Integration/DynamicDialogs/LDAPDialogs/Email?userid=#{userid}") 
  raise 'Error instantiating PScall' if data.blank? 
  req_email = $evm.root['Email']
  requester_email = requester.email || req_email ||nil

  # Get Owner Email else set to nil
  owner_email = miq_request.options[:owner_email] || nil
  $evm.log('info', "Requester email:<#{requester_email}> Owner Email:<#{owner_email}>")

  # if to is nil then use requester_email
  to = nil
  to ||= requester_email

  # If to is still nil use to_email_address from model
  to ||= $evm.object['to_email_address']

  # Get from_email_address from model unless specified below
  from = nil
  from ||= $evm.object['from_email_address']

  # Get signature from model unless specified below
  signature = nil
  signature ||= $evm.object['signature']

  # Build subject
  subject = "(#{miq_request.id})#{catalog_item} - Request APPROVED"

  # Build email body
  body = "Your request for #{catalog_item} was approved."
  body += "<br><br>Catalog Item: #{catalog_item}"
  body += "<br>RequestID: #{miq_request.id}"
  body += "<br>Status: Approved"
  body += "<br>Reason: #{miq_request.reason}"
  body += "<br><br>To view the Request <a href='https://#{appliance}/miq_request/show/#{miq_request.id}'>click here</a>"
  body += "<br><br> Thank you,"
  body += "<br> #{signature}"

  # Send email
  $evm.log("info", "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
  $evm.execute(:send_email, to, from, subject, body)
end

def emailapprover(miq_request, appliance)
  $evm.log('info', "Requester email logic starting")

  # Get requester object
  requester = miq_request.requester

  # Get requester email else set to nil
  requester_email = requester.email || nil

  # If to is still nil use to_email_address from model
  to = nil
  to ||= $evm.object['to_email_address']

  # Get from_email_address from model unless specified below
  from = nil
  from ||= $evm.object['from_email_address']

  # Get signature from model unless specified below
  signature = nil
  signature ||= $evm.object['signature']

  # Build subject
  subject = "Request ID #{miq_request.id} - Your Service provision request was Approved"

  # Build email body
  body = "Approver, "
  body += "<br>Service provision request received from #{requester_email} was approved."
  body += "<br><br>Approvers reason: #{miq_request.reason}"
  body += "<br><br>To view this Request go to: <a href='https://#{appliance}/miq_request/show/#{miq_request.id}'>https://#{appliance}/miq_request/show/#{miq_request.id}</a>"
  body += "<br><br> Thank you,"
  body += "<br> #{signature}"

  # Send email
  $evm.log("info", "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
  $evm.execute(:send_email, to, from, subject, body)
end

# Get miq_request from root
miq_request = $evm.root['miq_request']
dialog_options = miq_request['options'][:dialog]
raise "miq_request missing" if miq_request.nil?
$evm.log("info", "Detected Request:<#{miq_request.id}> with Approval State:<#{miq_request.approval_state}>")

# Override the default appliance IP Address below
appliance = nil
# appliance ||= 'evmserver.example.com'
# appliance ||= $evm.root['miq_server'].ipaddress
appliance = 'cloudforms'
catalog_item = $evm.root['service_template_provision_request'].source.name

# Email Requester
$evm.log('info', "catalog item name: #{$evm.root['service_template_provision_request'].source.name}")
catalog_items = ['Shiny App Creation', 'Shiny App Migration'] 
if catalog_items.include?($evm.root['service_template_provision_request'].source.name)
  if $evm.root['service_template_provision_request'].source.name == 'Shiny App Migration' 
     #Shiny Migration
     app_short_name = dialog_options['dialog_short_name']
     $evm.log("info", "AppShortName: #{app_short_name }")
     result = $evm.instantiate("/Integration/CommonMethods/PowerShell/PScall?script=C:\\vCO\\Scripts\\Cloudforms\\Shiny\\Get-TVAShinyAppOwner.ps1&value1=-AppShortName '#{app_short_name}'")
     raise 'Error instantiating PScall' if result.blank?
     $evm.log(:info, "result: #{result.inspect}")
     app_owner = $evm.root['ps_data'].match(/(^[a-zA-z0-9]{1,25}@tva.gov$)/).captures.first
     $evm.log(:info, "AppOwner: #{app_owner}") if @debug
     userid = $evm.root['user'].userid
     $evm.log(:info, "UserID: #{userid}") if @debug    
     data = $evm.instantiate("/Integration/DynamicDialogs/LDAPDialogs/Email?userid=#{userid}") 
     raise 'Error instantiating PScall' if data.blank? 
     app_requestor = $evm.root['Email']
     $evm.log(:info, "AppRequestor: #{app_requestor}") if @debug
     if app_owner == app_requestor
      $evm.log("info", "Exit ServiceTemplateProvisionRequest_Approved")
      exit MIQ_OK
     else
      $evm.log("info", "Email Requester")
      emailrequester(miq_request, appliance, catalog_item)
     end
   else
     $evm.log("info", "Email Requester")
     emailrequester(miq_request, appliance, catalog_item)
   end
else
  $evm.log("info", "Exit ServiceTemplateProvisionRequest_Approved")
  exit MIQ_OK
end
  
