
#
# Description: This method is used to email the requester and approver that the service request has been denied
#

def send_mail(to, from, subject, body)
  $evm.log(:info, "Sending email to #{to} from #{from} subject: #{subject}")
  $evm.execute(:send_email, to, from, subject, body)
end

def requester
  @miq_request.requester
end

def signature
  $evm.object['signature']
end

def reason
  @miq_request.reason
end

def approver_href(appliance)
  body = "<a href='https://#{appliance}/miq_request/show/#{@miq_request.id}'"
  body += ">click here</a>"
end

def approver_denied_text(requester_email, msg, reason, catalog_item)
  body = "The request received for #{catalog_item} was denied."
  body += "<br><br>Catalog Item: #{catalog_item}"
  body += "<br>RequestID: #{@miq_request.id}"
  body += "<br>Requestor: #{requester_email}"
  body += "<br>Status: Denied"
  body += "<br>Reason: #{reason}"
end

def approver_text(appliance, msg, requester_email, catalog_item)
  body = approver_denied_text(requester_email, msg, reason, catalog_item)
  body += "<br><br>To view the Request "
  body += approver_href(appliance)
  body += "<br><br> Thank you,"
  body += "<br> #{signature}"
end

def requester_email_address
  owner_email = @miq_request.options.fetch(:owner_email, nil)
  userid = $evm.root['user'].userid  
  data = $evm.instantiate("/Integration/DynamicDialogs/LDAPDialogs/Email?userid=#{userid}") 
  raise 'Error instantiating PScall' if data.blank? 
  req_email = $evm.root['Email']
  email = requester.email || req_email || owner_email || $evm.object['to_email_address']
  $evm.log(:info, "Requester email: #{email}")
  email
end

def email_approver(appliance, msg, approver_email, catalog_item)
  $evm.log(:info, "Approver email logic starting")
  requester_email = requester_email_address
  to = approver_email,$evm.object['to_email_address']
  from = $evm.object['from_email_address']
  subject = "(#{@miq_request.id})#{catalog_item}: Request DENIED"

  send_mail(to, from, subject, approver_text(appliance, msg, requester_email, catalog_item))
end

def requester_href(appliance)
  body = "<a href='https://#{appliance}/miq_request/show/#{@miq_request.id}'"
  body += ">click here</a>"
end

def requester_text(appliance, msg, catalog_item)
  body = "Your request for #{catalog_item} was denied."
  body += "<br><br>Catalog Item: #{catalog_item}"
  body += "<br>RequestID: #{@miq_request.id}"
  body += "<br>Status: Denied"
  body += "<br>Reason: #{reason}"
  body += "<br><br>To view the Request "
  body += requester_href(appliance)
  body += "<br><br> Thank you,"
  body += "<br> #{signature}"
end

def email_requester(appliance, msg, catalog_item)
  $evm.log(:info, "Requester email logic starting")
  to = requester_email_address
  from = $evm.object['from_email_address']
  subject = "(#{@miq_request.id})#{catalog_item}: Request DENIED"

  send_mail(to, from, subject, requester_text(appliance, msg, catalog_item))
end

@miq_request = $evm.root['miq_request']
dialog_options = @miq_request['options'][:dialog]
$evm.log(:info, "miq_request id: #{@miq_request.id} approval_state: #{@miq_request.approval_state}")
$evm.log(:info, "options: #{@miq_request.options.inspect}")

service_template = $evm.vmdb(@miq_request.source_type, @miq_request.source_id)
$evm.log(:info, "service_template id: #{service_template.id} service_type: #{service_template.service_type}")
$evm.log(:info, "description: #{service_template.description} services: #{service_template.service_resources.count}")

msg = @miq_request.resource.message || "Request denied"
#appliance = $evm.root['miq_server'].ipaddress
appliance = 'cloudforms'
catalog_item = $evm.root['service_template_provision_request'].source.name

# Approver Email
if $evm.root['service_template_provision_request'].source.name == 'Find VM'
  approver_email = 'tkopuru@tva.gov'
elsif $evm.root['service_template_provision_request'].source.name == 'Shiny App Creation'
  #approver_email = 'tkopuru@tva.gov'
  approver_email = 'mlkocoloski@tva.gov'
elsif $evm.root['service_template_provision_request'].source.name == 'Shiny App Migration'
  app_short_name = dialog_options['dialog_short_name']
  $evm.log(:info, "AppShortName: #{app_short_name}")
  result = $evm.instantiate("/Integration/CommonMethods/PowerShell/PScall?script=C:\\vCO\\Scripts\\Cloudforms\\Shiny\\Get-TVAShinyAppOwner.ps1&value1=-AppShortName '#{app_short_name}'")
  raise 'Error instantiating PScall' if result.blank?
  $evm.log(:info, "result: #{result.inspect}")
  approver_email = $evm.root['ps_data'].match(/(^[a-zA-z0-9]{1,25}@tva.gov$)/).captures.first
elsif $evm.root['service_template_provision_request'].source.name == 'Maximo Schema Refresh'
  approver_email = 'ncbrouhard@tva.gov','naloveless@tva.gov'
else
  approver_email = ''
end

email_requester(appliance, msg, catalog_item)
email_approver(appliance, msg, approver_email, catalog_item)
