#
# tva_arpcloud_provisioning.rb
#
# Handle TVA ARPcloud provisioning.
#
# dialog_option_1_XXX is used to pass parameter XXX and it's value directly to WinRM.
# dialog_option_2_XXX is to pass parameters to this method for deriving other parameters.
#
# Collect options and call PowerShell to do the deploy.
#

begin

  def log(level, msg, update_message = false)
    $evm.log(level, "#{msg}")
    @task.message = msg if @task && (update_message || level == 'error')
  end

  def yaml_data(option)
    @task.get_option(option).nil? ? nil : YAML.load(@task.get_option(option))
  end

  # check to ensure that dialog_parser has ran
  def parsed_dialog_information
    dialog_options_hash = yaml_data(:parsed_dialog_options)
    dialog_tags_hash = yaml_data(:parsed_dialog_tags)
    if dialog_options_hash.blank? && dialog_tags_hash.blank?
      log(:info, "Instantiating dialog_parser to populate dialog options")
      $evm.instantiate('/Service/Provisioning/StateMachines/Methods/DialogParser')
      dialog_options_hash = yaml_data(:parsed_dialog_options)
      dialog_tags_hash = yaml_data(:parsed_dialog_tags)
      raise 'Error loading dialog options' if dialog_options_hash.blank? && dialog_tags_hash.blank?
    end
    log(:info, "dialog_options_hash: #{dialog_options_hash.inspect}")
    log(:info, "dialog_tags_hash: #{dialog_tags_hash.inspect}")
    return dialog_options_hash, fix_dialog_tags_hash(dialog_tags_hash)
  end

  # fix_dialog_tags_hash to allow for support of Tag Control items in dialogs.  Previously, the Tag Control would
  # push values in as an array, which is not supported.  The fix_dialog_tag_hash parsed the dialog_tags_hash and changes
  # all array values to strings.
  def fix_dialog_tags_hash(dialog_tags_hash)
    unless dialog_tags_hash.empty?
      dialog_tags_hash.each do |build_num,build_tags_hash|
        build_tags_hash.each do |k,v|
          if v.is_a?(Array)
            log(:info, "fix_dialog_tags_hash: Build #{build_num}: updating key <#{k}> with array value <#{v}> to <#{v.first}>")
            build_tags_hash[k] = v.first if v.is_a?(Array)
          end
        end
      end
    end
    dialog_tags_hash
  end

  def images(offer,sku)
    require 'net/http'
    require 'json'
    url = "http://chapwcouchdb1.main.tva.gov:5984/azurecatalog/_design/catalog/_view/images?key=\[\"#{offer}\",\"#{sku}\"\]"
    uri = URI(url)
    response = Net::HTTP.get(uri)
    couch = JSON.parse response
    v={}
    couch['rows'].each { |x| v=x['value'] }
    return v
  end

  def publisher_friendlyname(publisher)
    require 'net/http'
    require 'json'
    url = "http://chapwcouchdb1.main.tva.gov:5984/azurecatalog/_design/catalog/_view/friendlynametopublisher?key=\"#{publisher}\""
    uri = URI(url)
    response = Net::HTTP.get(uri)
    couch = JSON.parse response
    p={}
    couch['rows'].each { |x| p=x['value'] }
    return p
  end

  @debug = true
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug

  @task = $evm.root['service_template_provision_task']
  @service = @task.destination
  log(:info, "Service: #{@service.name} id: #{@service.id} tasks: #{@task.miq_request_tasks.count}")

  dialog_options_hash, dialog_tags_hash = parsed_dialog_information

  # Massage the option_1 items into our arguments for PowerShell.
  ps_args = []
  dialog_options_hash[1].each do |k,v|
    ps_args << "-#{k} '#{v}'"
  end

  case dialog_options_hash[2][:Provider]
    when 'Azure'

      # Verify that name is not current in use.
      new_vm_name = dialog_options_hash[1][:Name]
      if new_vm_name.blank?
        raise 'name not found!'
      else
        existing_vm = $evm.vmdb(:vm_or_template).find_by_name(new_vm_name) rescue nil
        raise "vm_name #{new_vm_name} already exists!" unless existing_vm.blank?
      end

      offer = dialog_options_hash[1][:Offer]
      sku = dialog_options_hash[1][:SKU]
      publisher = dialog_options_hash[2][:Publisher]
      pub = publisher_friendlyname(publisher)
      email = $evm.root['user'].userid
      $evm.log(:info, "Email: #{email}") if @debug
    
      data = $evm.instantiate("/Integration/DynamicDialogs/LDAPDialogs/ntid?email=#{email}") 
      raise 'Error instantiating PScall' if data.blank?
      $evm.log(:info, "result: #{data.inspect}")
    
      ps_args << "-Version \'#{images(offer,sku)}\'"
      ps_args << "-Publisher \'#{pub}\'"
      if pub != "Redhat"
        join = "True"
      else
        join = "False"
      end
      ps_args << "-join #{join}"
      ps_args << "-vraUser #{$evm.root['userid'].split('@').first.split('-').first}"
      log(:info, "ps_args: #{ps_args.join(' ')}") if @debug
      #$evm.instantiate("/Integration/CommonMethods/Powershell/PScallLong?script=C:\\vCO\\Scripts\\Azure\\DeployAzureVM.ps1&value1=-Name #{azure_name} -vraUser #{username} -Publisher #{pub} -Offer #{offer} -SKU #{sku} -Version #{version} -vmSize #{azure_size} -join #{join}")
      #$evm.instantiate("/Integration/CommonMethods/Powershell/PScallLong?script=C:\\vCO\\Scripts\\Azure\\DeployAzureVM.ps1&value1=#{ps_args.join(' ')}")
      $evm.root['param11'] = "powershell_cmd=C:\\vCO\\Scripts\\Azure\\DeployAzureVM.ps1 #{ps_args.join(' ')}"
    
  when 'OnPremises'
#      citrix_name = dialog_options_hash[2][:CitrixName]
#      ecm_name = dialog_options_hash[2][:ECMName]
#      maximo_name = dialog_options_hash[2][:MaximoName]
#      default_name = dialog_options_hash[2][:DefaultName]
#      citrix_name ||= ''
#      ecm_name ||= ''
#      maximo_name ||= ''
#      default_name ||= ''

#      if citrix_name.length > 9 && citrix_name != '<None>'
#        server_name = citrix_name
#      elsif ecm_name.length > 2 && ecm_name != '<None>'
#        server_name = ecm_name
#      elsif maximo_name.length > 3 && maximo_name != '<None>'
#        server_name = maximo_name
#      elsif default_name.length > 3 && default_name != '<None>'
#        server_name = default_name
#      else
#        raise "Failed to find a name"
#      end
#      $evm.log(:info, "server_name: #{server_name}") if @debug
#      ps_args << "-Name #{server_name}"
#      # Store our target vm name back into the dialog for next methods in the state machine.
#      $evm.set_state_var(:dialog_option_1_name, server_name)
      email = $evm.root['user'].userid
      $evm.log(:info, "Email: #{email}") if @debug
    
      data = $evm.instantiate("/Integration/DynamicDialogs/LDAPDialogs/ntid?email=#{email}") 
      raise 'Error instantiating PScall' if data.blank?
      $evm.log(:info, "result: #{data.inspect}")
    
      ps_args << "-ntid #{$evm.root['userid'].split('@').first.split('-').first}"
      log(:info, "ps_args: #{ps_args.join(' ')}") if @debug
      #$evm.instantiate("/Integration/CommonMethods/Powershell/PScallLong?script=C:\\vCO\\Scripts\\ARPNET\\ArpCloud\\ArpCloud-Deploy.ps1&value1=-Name #{server_name} -CPU #{onprem_cpu} -RAM #{onprem_ram} -Template #{onprem_template} -NTID #{username}")
      #$evm.instantiate("/Integration/CommonMethods/Powershell/PScallLong?script=C:\\vCO\\Scripts\\ARPNET\\ArpCloud\\ArpCloud-Deploy.ps1&value1=#{ps_args.join(' ')}")
      $evm.root['param11'] = "powershell_cmd=C:\\vCO\\Scripts\\ARPNET\\ArpCloud\\ArpCloud-Deploy.ps1 #{ps_args.join(' ')}" 
  end

rescue => err
  log(:error, "[#{err}]\n#{err.backtrace.join("\n")}")
  @task['status'] = 'Error' if @task
  @task.finished("#{err}") if @task
  @service.remove_from_vmdb if @service
  exit MIQ_ABORT
end
