# Author: Jeffrey Cutter (jcutter@redhat.com)
#
# Description: Call a vRA workflow to perform retirement.
#

begin

  require 'rest_client'
  require 'json'

  def call_api(payload, guid)

    # get the server fqdn/ip and port number to send the request to
    server      = $evm.object['hostname']
    username    = $evm.object['username']
    password    = $evm.object.decrypt('password')
    $evm.log(:info, "server: #{server}, username: #{username}") if @debug

    # build the URL for the REST request
    url = "https://#{server}:8281/vco/api/workflows/#{guid}/executions"

    $evm.log(:info, "Call vRA API URL: <#{url}>") if @debug
    $evm.log(:info, "payload: #{payload.inspect}") if @debug

    # build the REST request
    params = {
        :method   => :post,
        :url      => url,
        :user     => username,
        :password => password,
        :headers  => {
            :content_type  => :json,
            :accept        => :json
        },
        :timeout => 90,
        :payload => payload,
        :verify_ssl  => false
    }

    # call the REST service
    begin
      response = RestClient::Request.new(params).execute
    # $evm.set_state_var('vm_removed_from_provider', true)
    rescue => e
      $evm.log(:error, "REST response: #{e.response.body.inspect}") if @debug
      raise e
    end

    unless response.code == 200 || response.code == 201 || response.code == 202
      $evm.log(:error, "Error calling REST API")
      $evm.log(:info, "Response Code: <#{response.code}>  Response: <#{response.inspect}>")
    end
    $evm.log(:info, "API Response Data: #{response.inspect}") if @debug
  end

  #######
  # BEGIN HERE
  #######

  @debug = true
  $evm.root.attributes.sort.each { |k, v| $evm.log(:info, "root: #{k}: #{v}") } if @debug
  $evm.object.attributes.sort.each { |k, v| $evm.log("info", "object: #{k}: #{v}") } if @debug

  guid = 'a67394fc-fcdb-4c52-b154-335a5fd2fd4d'
  server_name = $evm.root['dialog_server_name']
  cycle_id = $evm.root['dialog_cycle_id']
  server_type = $evm.root['dialog_server_type']
  os = $evm.root['dialog_os']
  sql = $evm.root['dialog_sql']
    
  email = $evm.root['user'].userid
  $evm.log(:info, "Email: #{email}") if @debug    
  data = $evm.instantiate("/Integration/DynamicDialogs/LDAPDialogs/ntid?email=#{email}") 
  raise 'Error instantiating PScall' if data.blank? 
  username = $evm.root['userid'].split('@').first.split('-').first
  $evm.log(:info, "Username: #{username}") if @debug
  
  raise 'Required parameter not found!' if server_name.blank? || cycle_id.blank? || server_type.blank? || os.blank? || sql.blank? || username.blank?

  payload = {"parameters":
      [
                {
                                "value": {"string":{ "value": server_name}},
                                "type": "string",
                                "name": "serverName",
                                "scope": "local"
                },
                {
                                "value": {"string":{ "value": cycle_id}},
                                "type": "string",
                                "name": "CycleID",
                                "scope": "local"
                },
                {
                                "value": {"string":{ "value": server_type}},
                                "type": "string",
                                "name": "serverType",
                                "scope": "local"
                },
                {
                                "value": {"string":{ "value": os}},
                                "type": "string",
                                "name": "OS",
                                "scope": "local"
                },
                {
                                "value": {"string":{ "value": sql}},
                                "type": "string",
                                "name": "SQL",
                                "scope": "local"
                },
                {
                                "value": {"string":{ "value": username}},
                                "type": "string",
                                "name": "NTID",
                                "scope": "local"
                }
      ]
  }.to_json

  call_api(payload, guid)

  exit MIQ_OK

rescue => err
  $evm.log(:error, "[(#{err.class})#{err}]\n#{err.backtrace.join("\n")}")
  exit MIQ_ABORT
end
