#
# Description: This method is used to add 14 days to retirement date when target
# VM has a retires_on value and is not already retired
#

# Number of days to automatically extend retirement
vm_retire_extend_days = nil
vm_retire_extend_days ||= $evm.object['vm_retire_extend_days'].to_i
raise "ERROR - vm_retire_extend_days not found!" if vm_retire_extend_days.nil?

$evm.log("info", "Number of days to extend: <#{vm_retire_extend_days}>")

obj = $evm.object("process")

vm = obj["vm"] || $evm.root["vm"]
vm_name = vm.name

# Bail out if VM does not have retirement date
if vm.retires_on.nil? | vm.retires_on == ""
  $evm.log("info", "VM '#{vm_name}' has no retirement date - extension bypassed")
  exit MIQ_OK
end

# If VM is already retired to do not continue
if vm.retired
  $evm.log("info", "VM '#{vm_name}' is already retired. retires_on date: #{vm.retires_on}. No Action taken")
end

$evm.log("info", "VM: <#{vm_name}> current retirement date is #{vm.retires_on}")

unless vm.retires_on.nil?
  if vm.tagged_with?("lifecycle", "soft_retire")
    $evm.log("info", "Extending retirement <#{vm_retire_extend_days}> days for VM: <#{vm_name}>")

    # Set new retirement date here
    vm.retires_on = Time.now + vm_retire_extend_days.days

    $evm.log("info", "VM: <#{vm_name}> new retirement date is #{vm.retires_on}")

    # Resetting retirement warning
    $evm.log("info","Retirement Warning reset for VM: <#{vm.name}>")
    vm.retirement_warn = 14
    
  # vm.retirement[:last_warn] = nil

  # $evm.log("info", "Inspecting retirement: <#{vm.retirement.inspect}>")
  elsif vm.tagged_with?("lifecycle", "retire_full")
    $evm.log("info", "Extending retirement <#{vm_retire_extend_days}> days for VM: <#{vm_name}>")

    # Set new retirement date here
    vm.retires_on = Time.now + vm_retire_extend_days.days

    $evm.log("info", "VM: <#{vm_name}> new retirement date is #{vm.retires_on}")
    
    # Power On the VM
    $evm.log('info', "Powering On VM <#{vm.name}>")
    state = vm.attributes['power_state']
    if %w(off).include?(state)
      $evm.log('info', "Powering Off VM <#{vm.name}>")
      vm.start
    end 
    
    # Tag VM with Soft Retire
    $evm.log("info", "VM: <#{vm_name}> is tagged")
    vm.tag_unassign('lifecycle/retire_full')
    vm.tag_assign('lifecycle/soft_retire')

    # Resetting retirement warning
    $evm.log("info","Retirement Warning reset for VM: <#{vm.name}>")
    vm.retirement_warn = 14
    
  # vm.retirement[:last_warn] = nil

  # $evm.log("info", "Inspecting retirement: <#{vm.retirement.inspect}>")
  else
    $evm.log("info", "Extending retirement <#{vm_retire_extend_days}> days for VM: <#{vm_name}>")

    # Set new retirement date here
    vm.retires_on = Time.now + vm_retire_extend_days.days

    $evm.log("info", "VM: <#{vm_name}> new retirement date is #{vm.retires_on}")

    # Resetting retirement warning
    $evm.log("info","Retirement Warning reset for VM: <#{vm.name}>")
    vm.retirement_warn = 2
    
  # vm.retirement[:last_warn] = nil

  # $evm.log("info", "Inspecting retirement: <#{vm.retirement.inspect}>")
  end

  ######################################
  #
  # VM Retirement Exended Email
  #
  ######################################

  # Get VM Owner Name and Email
  evm_owner_id = vm.attributes['evm_owner_id']
  owner = nil
  owner = $evm.vmdb('user', evm_owner_id) unless evm_owner_id.nil?
  $evm.log("info", "Inspecting VM Owner: #{owner.inspect}")
  
  userid = owner.userid
  $evm.log(:info, "UserID: #{userid}")     
  data = $evm.instantiate("/Integration/DynamicDialogs/LDAPDialogs/Email?userid=#{userid}") 
  raise 'Error instantiating PScall' if data.blank? 
  email = $evm.root['Email']
  $evm.log(:info, "Email: #{email}") 
  

  # to_email_address from owner.email then from model if nil
  if email
    to = email.sub('-wa@tva.gov','@tva.gov'),$evm.object['to_email_address']
  else
    to = $evm.object['to_email_address']
  end

  # Get from_email_address from model unless specified below
  from = nil
  from ||= $evm.object['from_email_address']

  # Get signature from model unless specified below
  signature = nil
  signature ||= $evm.object['signature']

  # email subject
  subject = "VM Retirement Extended for #{vm_name}"

  # Build email body
  body = "Hello, "
  body += "<br><br>The retirement date for your virtual machine: [#{vm_name}] has been extended to: [#{(vm.retires_on).in_time_zone("EST")}]."
  body += "<br><br> Thank you,"
  body += "<br> #{signature}"

  #
  # Send email
  #
  $evm.log("info", "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
  $evm.execute('send_email', to, from, subject, body)
end
