#
# Description: This method sends out retirement emails when the following events are raised:
# Events: vm_retire_warn, vm_retired, vm_entered_retirement
# Model Notes:
# 1. to_email_address - used to specify an email address in the case where the
#    vm's owner does not have an  email address. To specify more than one email
#    address separate email address with commas. (I.e. admin@example.com,user@example.com)
# 2. from_email_address - used to specify an email address in the event the
#    requester replies to the email
# 3. signature - used to stamp the email with a custom signature
#

# multiple warning emails.
# Set the number of days from the retirement date for the next warning.
def update_retirement_warning(vm)
  # Get the warning days from the current object
  warn_days_1 = $evm.object['warn_days_1'] || 14
  warn_days_2 = $evm.object['warn_days_2'] || 2

  # Get the number of days warning for the vm that is currently set.
  current_retire_days = vm.retirement_warn

  $evm.log("info", "Found VM: <#{vm.name}> has current_retire_days of #{current_retire_days}")
  
  if current_retire_days > warn_days_1
    vm.retirement_warn = warn_days_1
  elsif current_retire_days == warn_days_1 || current_retire_days > warn_days_2
    vm.retirement_warn = warn_days_2
  else
    vm.retirement_warn = nil
  end
 
  $evm.log("info","VM: <#{vm.name}> Retirement Warning reset to <#{vm.retirement_warn}>")
end  

# Look in the current object for a VM
vm = $evm.object['vm']
if vm.nil?
  vm_id = $evm.object['vm_id'].to_i
  vm = $evm.vmdb('vm', vm_id) unless vm_id == 0
end

# Look in the Root Object for a VM
if vm.nil?
  vm = $evm.root['vm']
  if vm.nil?
    vm_id = $evm.root['vm_id'].to_i
    vm = $evm.vmdb('vm', vm_id) unless vm_id == 0
  end
end

# Look in the Root Object for a Provision/Request
prov = $evm.root['miq_provision_request'] || $evm.root['miq_provision']
vm = prov.vm if prov && vm.nil?

raise "User not specified" if vm.nil?

# Get VM Name
vm_name = vm['name']

# Look at the Event Type in the Current Object or in the Root Object
event_type = $evm.object['event'] || $evm.root['event_type']

# Get VM Owner Name and Email
evm_owner_id = vm.attributes['evm_owner_id']
owner = nil
owner = $evm.vmdb('user', evm_owner_id) unless evm_owner_id.nil?
$evm.log("info", "Inspecting VM Owner: #{owner.inspect}")
  
userid = owner.userid
$evm.log(:info, "UserID: #{userid}")     
data = $evm.instantiate("/Integration/DynamicDialogs/LDAPDialogs/Email?userid=#{userid}") 
raise 'Error instantiating PScall' if data.blank? 
email = $evm.root['Email']
$evm.log(:info, "Email: #{email}") 
  

# to_email_address from owner.email then from model if nil
if event_type == "vm_retire_warn" && vm.tagged_with?("tva_automation", "lab_retirement")
  if email
    to = email.sub('-wa@tva.gov','@tva.gov'),'ITEngLabAutomationTeam@tva.gov'
  else
    to = $evm.object['to_email_address']
  end  
else  
  if email
    to = email.sub('-wa@tva.gov','@tva.gov'),$evm.object['to_email_address']
  else
    to = $evm.object['to_email_address']
  end
end
######################################
#
# VM Retirement Warning Email
#
######################################
if event_type == "vm_retire_warn" && vm.tagged_with?("lifecycle", "soft_retire")

  # Get from_email_address from model unless specified below
  from = nil
  from ||= $evm.object['from_email_address']

  # Get signature from model unless specified below
  signature = nil
  signature ||= $evm.object['signature']

  # email subject
  subject = "VM Retirement Warning for #{vm_name}"

  # Build email body
  body = "Hello, "
  body += "<br><br>Your virtual machine: [#{vm_name}] will be powered off on [#{vm['retires_on'].in_time_zone("EST")}]."
  body += "<br><br>If you need to use this virtual machine past this date please request an extension by following the direction at <a href='http://rtd.tva.gov/Compute/Documentation/InnovationCenter/ExpirationsAndRenewals.html'>this link</a> under the 'OnPremises' section."
  body += "<br><br>&quot;https://cloudforms/vm_infra/explorer&quot;"
  body += "<br><br> Thank you,"
  body += "<br> #{signature}"
  
  update_retirement_warning(vm)
end

if event_type == "vm_retire_warn" && vm.tagged_with?("lifecycle", "retire_full")

  # Get from_email_address from model unless specified below
  from = nil
  from ||= $evm.object['from_email_address']

  # Get signature from model unless specified below
  signature = nil
  signature ||= $evm.object['signature']

  # email subject
  subject = "VM Retirement Warning for #{vm_name}"

  # Build email body
  body = "Hello, "
  body += "<br><br>Your virtual machine: [#{vm_name}] will be deleted on [#{vm['retires_on'].in_time_zone("EST")}]."
  body += "<br><br>If you need to use this virtual machine past this date please request an extension."
  body += "<br><br>&quot;https://cloudforms/vm_infra/explorer&quot;"
  body += "<br><br> Thank you,"
  body += "<br> #{signature}"
end

$evm.log("info", "Sending email to <#{to}> from <#{from}> subject: <#{subject}>")
$evm.execute('send_email', to, from, subject, body)
